/*
  RT-11 file image manipulation program
 
  from "RT-11 Adapter Package for CP/M" by William C. Colley, III

	Rev. 1.0 -- July 1980

	Rev. 1.1 -- March 1981 consisting of adding a valid system date
			word to all files placed on the RT-11 disk and
			putting the volume ID on a disk when the directory
			is initialized.  This will keep RT-11 versions
			later than V02C from choking.

	copyright (c) 1980, William C. Colley, III

	Rev. 1.2 -- Craig Davenport - Incitec Ltd      (Feb 1984)
				      P O Box 140
				      Morningside,
				      Qld 4170,
				      Australia.
		 -- Modified for Digital Research C compiler under CP/M-86
                 -- Assebmbly language routines added for BIOS calls etc.

	Rev. 1.3 -- port to linux brad@heeltoe.com
		added "adaption" code to fake out the bios calls with
		a local unix file

  The main-line routine is here, as are the individual command processors.
*/

#include "rt11.h"

#include <stdio.h>
#include <ctype.h>
#include <time.h>

/*
Main program..........
*/

void
get_sysdate(void)
{
	int  m, d, y;
	time_t t;
	struct tm *tmp;

	/* Get system date word for later RT-11 versions. */
	t = time(NULL);
	tmp = localtime(&t);
	m = tmp->tm_mon;
	d = tmp->tm_mday;
	y = tmp->tm_year - 1900;

	sysdate = (m << 10) + (d << 5) + (y - 0110);
}

void
usage(void)
{
	puts("RT-11 File I/O Package -- Rev. 1.3 -- Apr 2004\n");
	printf("usage:\n");
	printf("-d              Directory list\n");
	printf("-e <name>       Erase file on RT-11 disk\n");
	printf("-g <name>       Get file from RT-11 disk\n");
	printf("-i              Initialize RT-11 disk\n");
	printf("-p <name>       Put file onto RT-11 disk\n");
	printf("-r <from> <to>  Rename file on RT-11 disk\n");
	printf("-t <name>       Type file on RT-11 disk\n");
	printf("\n");
	printf("-f <file>       Specify RT-11 volume\n");
	
	printf("\n");
}

extern char *optarg;
extern int optind;

main(int argc, char *argv[])
{
	int c;
	char *secondarg;

	if (argc < 2) {
		usage();
		exit(1);
	}

	get_sysdate();

	while ((c = getopt(argc, argv, "de:g:ipr:t:f:")) != -1) {
		switch (c) {
		case 'd':	list_dir();		break;
		case 'e':	erase_file(optarg);	break;
		case 'g':	get_file(optarg);	break;
		case 'i':	init_disk();		break;
		case 'p':	put_file(optarg);	break;
		case 'r':
			if (optind >= argc) {
				usage();
				exit(3);
			}
			secondarg = argv[optind];

			rename_file(optarg, secondarg);
			break;
		case 't':	type_file(optarg);	break;
		case 'f':
			if (BIOS_OPEN(optarg)) {
				usage();
				exit(2);
			}
		}
	}

	exit(0);
}

/*
Routine to erase a file on the RT-11 disk.
*/

erase_file(char *target_name)
{
	int file_name[3];

	if (!name_to_rdx50(target_name, file_name))
		return;

	if (!delete(file_name))
		printf("error: file '%s' does not exist\n", target_name);
}

/*
Routine to rename a file on the RT-11 disk.
*/

rename_file(char *old_target, char *new_target)
{
	int old_name[3], new_name[3];

	if (!name_to_rdx50(old_target, old_name))
		return;
	if (!name_to_rdx50(new_target, new_name))
		return;

	if (!rrename(old_name,new_name)) {
		printf("error: file '%s' does not exist\n", old_target);
	}
}

/*
Routine to list the directory on the RT-11 disk.
*/

list_dir()
{
	int t;
	char s[4], volume_id[512], temp[13];
	temp[12] = '\0';

	emt_375(BIOSREAD,1,1,volume_id);

	printf("Volume: ");
	putstr(temp,&volume_id[236*2]);
	printf("%s\n", temp);
	printf(" Owner: ");
	putstr(temp,&volume_id[242*2]);
	printf("%s\n", temp);

	usrcom();
	puts("\nName  .Ext   Blks    Date     Start    Extras\n");
	do
	{
		while(*(dir_pointer + 1) != ENDSEG)
		{
			if (*(dir_pointer + 1) == PERM)
			{
				print_name(dir_pointer + 2);
				printf("   %3o   ",_getword(dir_pointer,4));
				if (t = _getword(dir_pointer,6) & 0x7fff)
				{
					getmon(t >> 10,s);
					printf("%2d-%s-%2d   ",(t >> 5) & 037,
						s,(t & 037) + 0110);
				}
				else puts("            ");
				printf("%3o   ",file_start);
				for (t = 0; t < directory.extra_bytes; t++)
					printf("  %3o",*(dir_pointer+t+14));
			}
			else printf("< unused >   %3o               %3o",
				_getword(dir_pointer,4),file_start);
			incr1();
			putchar('\n');
		}
	}
	while (nxblk());
}

/*
Function to get a file from the RT-11 disk onto the CP/M disk.
*/

get_file(char *target_name)
{
	char CP_file[1024], xfer_buffer[512];
	int t, RT_file[3], fd;

	if (!name_to_rdx50(target_name, RT_file))
		return;

	CP_file[0] = 0;

	if (CP_file[0] == '\0') sprint_name(RT_file,CP_file);
	fix_spaces(CP_file);
	if (0) printf("CP_file '%s'\n", CP_file);

	if (!lookup(RT_file))
	{
		printf("error: no source file\n");
		return;
	}

	if ((fd = creat(CP_file, 0666)) == -1)
	{
		perror(CP_file);
		return;
	}

	for (t = _getword(dir_pointer,4); t > 0; t--)
	{
		emt_375(BIOSREAD,file_start++,1,xfer_buffer);
		if (write(fd,xfer_buffer,512) != 512)
		{
			perror(CP_file);
			close(fd);
			return;
		}
	}
	close(fd);
}

/*
Routine to type an RT-11 file on the console.
*/

type_file(char *target_name)
{
	char c, xfer_buffer[512];
	int file_name[3], i, j;

	if (!name_to_rdx50(target_name, file_name))
		return;

	if (!lookup(file_name))
	{
		printf("error: file '%s' does not exist\n", target_name);
		return;
	}
	for (i = _getword(dir_pointer,4); i > 0; i--)
	{
		emt_375(BIOSREAD,file_start++,1,xfer_buffer);
		for (j = 0; j < 512; j++)
		{
			if ((c = xfer_buffer[j]) == '\0' || c == ('Z' - 64))
				return;
			putchar(c);
		}
	}
}

/*
Routine to put a file onto the RT-11 disk.
*/

put_file(char *target_name)
{
	int RT_file[3], fd, size,numbyte;
	char CP_file[1024], xfer_buffer[512];

	strcpy(CP_file, target_name);
	if (!name_to_rdx50(target_name, RT_file))
		return;

	if ((fd = open(CP_file,0)) == -1)
	{
		printf("error: could not open source file '%s'\n",
		       target_name);
		return;
	}
	size = filesize(CP_file);
	if ((file_start = enter(RT_file,size)) == 0) return;
	while (--size >= 0)
	{
		if ((numbyte=read(fd,xfer_buffer,512)) == -1)
		{
			perror("read");
			close(fd);
			return;
		}
		if (numbyte != 512)				/* eof */
			memset(xfer_buffer+numbyte,'Z'-64,512-numbyte);
		emt_375(WRITE,file_start++,1,xfer_buffer);
	}
	close(fd);
	klose(RT_file);
}

/*
Routine to initialize the directory on an RT-11 disk.
*/

init_disk()
{
	char temp[50];
	unsigned volume_id[256];

#if 0
	puts("Are you sure you want to do this (Y/N)? ");
	gets(temp);
	if (temp[0] != 'Y')
	{
		puts("\t.....Aborted\n");
		return;
	}
#endif

	memset(&directory,0,1024);
	puts("Number of directory segments (1 to 31)? ");
	while (1)
	{
		scanf("%d",&directory.total_segments);
		if (directory.total_segments > 0 &&
			directory.total_segments < 32) break;
		puts("Say again? ");
	}

	puts("Extra bytes per directory entry? ");
	scanf("%d",&directory.extra_bytes);

	directory.highest_segment = 1;
	directory.first_block = (directory.highest_segment + 1) * 2 + 4;
	directory.entries[1] = EMPTY;
	_putword(directory.entries,4,488 - 2 * directory.total_segments);
	directory.entries[15 + directory.extra_bytes] = ENDSEG;
	segrw(WRITE,1);
	memset(volume_id,' ',512);
	volume_id[0] = 0;
	volume_id[233] = 1;
	volume_id[234] = 6;
	volume_id[235] = 0107251;
	temp[0] = 0;

	puts("Volume ID (CR for default)? ");
	gets(temp);
	gets(temp);
	putstr(&volume_id[236], temp[0]==0 ? "RT11A" : temp);
	puts("Owner name (CR for default)? ");
	gets(temp);
	if (temp[0] != 0) 
	    putstr(&volume_id[242], temp);
 	putstr(&volume_id[248], "DECRT11A");
	emt_375(WRITE,1,1,volume_id);

	puts("\t.....Done\n");
}

/*
Internal function in initialize directory routine to put up to 12 characters
from a string into the core image of the volume ID block (block 1 on the disk).
Note that this routine is an addition at Ver 1.0.
*/

putstr(buffer,string)
char *buffer, *string;
{
	char i;
	for (i = 12; i > 0 && *string != '\0'; i--)
	{
		*buffer = toupper(*string);
		buffer+=1;
		string+=1;
	}
}

